using System;

namespace LinearAlgebra
{
	/// <summary>
	/// A vector of arbitrary size.
	/// </summary>
	public struct VectorN
	{
		private double[] v;

		public VectorN(int size)
		{
			v = new double[size];
		}

		public VectorN(params double[] coords)
		: this(ref coords)
		{}

		public VectorN(ref double[] coords)
		{
			v = coords;
		}

		public int Size
		{
			get
			{
				return v.Length;
			}
		}

		public double Get(int i)
		{
			return v[i];
		}

		public void Set(int i, double value)
		{
			v[i] = value;
		}

		public double Length()
		{
			double s = 0.0;
			foreach(double d in v)
			{
				s += d*d;
			}
			return Math.Sqrt(s);
		}

		private static void checkDimensions(ref VectorN v1, ref VectorN v2)
		{
			if(v1.Size != v2.Size)
			{
				throw new Exception("Dimension mismatch.");
			}
		}

		public static VectorN operator + (VectorN v1, VectorN v2)
		{
			checkDimensions(ref v1, ref v2);
			double[] v = new double[v1.Size];
			for(int i = 0; i < v1.Size; i++)
			{
				v[i] = v1.v[i] + v2.v[i];
			}
			return new VectorN(ref v);
		}

		public static VectorN operator - (VectorN v1, VectorN v2)
		{
			checkDimensions(ref v1, ref v2);
			double[] v = new double[v1.Size];
			for(int i = 0; i < v1.Size; i++)
			{
				v[i] = v1.v[i] - v2.v[i];
			}
			return new VectorN(ref v);
		}

		public static VectorN operator * (double f, VectorN v1)
		{
			double[] w = (double[])v1.v.Clone();
			for(int i = 0; i < v1.Size; i++)
			{
				w[i] *= f;
			}
			return new VectorN(ref w);
		}

		public static VectorN operator * (VectorN v1, double f)
		{
			return f*v1;
		}

		public static VectorN operator / (VectorN v1, double f)
		{
			double[] w = (double[])v1.v.Clone();
			for(int i = 0; i < v1.Size; i++)
			{
				w[i] /= f;
			}
			return new VectorN(ref w);
		}
	}
}
